% Estimating the image sensor noise of a single noisy raw image.
%
% [est_a, est_b] = NoiseEstCWLS(noisy_image, patchSize, quant )
%
%-Output parameters
% est_a: estimated parameter a in Poisson-Gaussian noise model. 
% est_b: estimated parameter b in Poisson-Gaussian noise model.
% 
%-Input parameters
% noisy_image: input noisy image (in the range [0,1])
% patchSize: patch size (default is 5)
% quant: quantile (default is 0.5)
%
%-Example:
% img = im2dobule(imread('noisy_img.png'));
% [est_a, est_b] = NoiseLevelEst(img);
%
%-Reference:
% [1] Li Dong, Jiantao Zhou, Yuan Yan Tang, 
% "Effective and Fast Estimation for Image Sensor Noise via Constrained Weighted Least Squares",
% IEEE Transaction on Image Processing (T-IP), Accepted, Mar. 2018.

function [est_a, est_b, p] = NoiseEstCWLS(noisy_image, patchSize, quant )

    warning('off','all');

    if nargin < 3
        patchSize = 5;
        quant = 0.5; 
    end
    DD = generateDD(patchSize);
    
    % ==== for image in the range [0,1] ==== 
    bitdepth = 1;
    qstep = 1/255;    
    
    % ==== sliding image ====
    vec_image = im2colstep( noisy_image, [patchSize patchSize], [1 1]);
    
    % ==== compute the mean of each patch ====
    mean_vec_image = (mean(vec_image,1));

    [bincnt, ~] = histc(mean_vec_image,0:qstep:bitdepth);
    [~, intensities] = sort(bincnt,'descend');

   temp = bincnt; temp(temp==0) =[];
   N = sum(bincnt(intensities)>quantile(temp,quant) & bincnt(intensities)>30); 
   xbars = intensities(1:N)*qstep;
    
    % ==== DCT basis filters =====
    W = DCTBasis(patchSize);
    

    yVars = zeros(N,1);
    xBars = zeros(N,2);
    p = zeros(size(vec_image,2),1);
    tconst = trace(DD);
    weight = zeros(N,1);
    shape = patchSize*(patchSize -2)/2; % filter [-1/2 0 1/2]&[-1/2 0 1/2]
    
    for k = 1:N
        xbar = xbars(k);
        
        % select the patch with similar mean, with the allowed variantion
        select_patch = (mean_vec_image>=xbar-qstep/2 & mean_vec_image<=xbar+qstep/2);
        xbar = mean(mean_vec_image(select_patch));
  
        select_vec_image = vec_image(:,select_patch) - xbar;

        % ========== compute the weight using texture stregnth ==========
        select_strength = sum( (select_vec_image'*DD).*select_vec_image',2);
        
        
        % ========== each row of W represents a band filter ==========
        yVars(k)= mean(var(W(end-2:end,:) * select_vec_image,[],2)); 
        
        xBars(k,:) = [xbar,1];
        
        % ========= Gamma distribution ==============
        scale = 2* yVars(k).*tconst/(patchSize* (patchSize -2));% filter [-1/2 0 1/2]
        

        % ========= likelihood =========
        likelihood = select_strength.^(shape-1).*exp(-select_strength/scale)./scale^shape; % speedup gampdf by closed-form

        likelihood(isnan(likelihood)) = [];
        weight(k) = median(likelihood);
        
    end

    indx = isnan(xBars(:,1));
    xBars(indx,:) =[];
    yVars(indx)= [];
    weight(indx) = [];
    
    % ==== only ||W(y-Ax)||^2_2  ===
    weight = weight/sum(weight);
    W = diag(weight);

    opts = optimset('Display','off');
    est = lsqlin(W*xBars,W*yVars,[],[],[],[],[eps,eps],Inf,xBars\yVars,opts);
    est_a = est(1);
    est_b = est(2);


end


